<?php
/**
 * @file
 * Angil module drush integration.
 */

/**
 * Implements hook_drush_command().
 */
function angil_drush_command() {
  $items = array();

  $items['angil-node-clone'] = array(
    'description' => 'Clone node by specified nids. ',
    'aliases' => array('anc'),
    'examples' => array(
      'drush angil-node-clone 7' => 'Clone node with nid one time.',
      'drush anc 3,15 10' => 'Clone nodes with nid 3 and 15 ten times.',
    ),
  );

  $items['angil-node-delete'] = array(
    'description' => 'Immidiatly delete nodes by nids.',
    'aliases' => array('andel'),
    'examples' => array(
      'drush angil-node-delete 7' => 'Delete node with nid 7.',
      'drush and 3,15' => 'Delete nodes with nid 3 and 15.',
    ),
  );

  $items['angil-node-publish'] = array(
    'description' => 'Publish nodes.',
    'aliases' => array('anpub'),
    'examples' => array(
      'drush anpub 7' => 'Publish node with nid 7.',
      'drush angil-node-publish 3,15' => 'Publish nodes with nid 3 and 15.',
    ),
  );

  $items['angil-node-unpublish'] = array(
    'description' => 'Unpublish nodes.',
    // @todo there is a bad alias, need better.
    'aliases' => array('anunpub'),
    'examples' => array(
      'drush anpub 7' => 'Unpublish node with nid 7.',
      'drush angil-node-publish 3,15' => 'Unpublish nodes with nid 3 and 15.',
    ),
  );

  $items['angil-node-dump'] = array(
    'description' => 'Dump to console node object.',
    'aliases' => array('and'),
    'examples' => array(
      'drush andump 7' => 'Var_dump node with nid 7.',
    ),
  );

  $items['angil-time-to-date'] = array(
    'description' => 'Convert timestamp to date and print it.',
    'aliases' => array('attd'),
    'examples' => array(
      'drush attd 1309665756' => 'Print date representation of 1309665756 time',
    ),
  );

  $items['angil-clear-content-cache'] = array(
    'description' => 'Clear content cache (panes, views, etc..). Usefull shortcut of admin_menu cache flush.',
    'aliases' => array('accc'),
    'examples' => array(
      'drush accc' => 'Clear content cache (panes, views, etc..).',
    ),
  );

  $items['angil-get-theme-preprosseces'] = array(
    'description' => 'Get theming preprocesses, ordered like will be called.',
    'aliases' => array('agtp'),
    'examples' => array(
      'drush agtp page' => 'Get preprocess for "page" theming.',
    ),
  );

  $items['angil-compare-module-weight'] = array(
    'description' => 'Compare weight of specified modules.',
    'aliases' => array('acmw'),
  );

  $items['angil-vardump'] = array(
    'description' => 'Evaluate input string and var_dump results.',
    'aliases' => array('ava'),
    'examples' => array(
      'drush ava "node_load(13)"' => 'Var_dump object of loaded node.',
    ),
  );

  $items['angil-check-system-table'] = array(
    'description' => 'During development in system table could records from deleted modules, etc.',
    'aliases' => array('acst'),
  );

  $items['angil-get-flat-reflection'] = array(
    'description' => 'Return string, flat representation of specified function. Usefull for debug on remote system.',
    'aliases' => array('agfr'),
    'examples' => array(
      'drush agfr --func="node_save"' => 'Print content of node_save function.',
    ),
  );

  $items['angil-path-check'] = array(
    'description' => 'Check for system path existance.',
    'aliases' => array('apc'),
    'options' => array(
      '--item' => 'Return menu item if succeed.',
    ),
    'examples' => array(
      'drush apc "node/1"' => 'Check drupal system path "node/1".',
    ),
  );

  $items['angil-admin-account'] = array(
    'description' => 'Set username and password for uid = 1.',
    'aliases' => array('aaa'),
    'examples' => array(
      'drush aaa admin admin' => 'Set admin/admin for uid = 1.',
    ),
  );

  return $items;
}

/**
 * Drush callback
 * Clone node by specified nid
 */
function drush_angil_node_clone() {
  $modify_field = drush_get_option('modify', FALSE);
  $args = drush_get_arguments();
  // Skip op name.
  array_shift($args);
  $new_nids = array();
  if (!empty($args)) {
    $nids = explode(',', array_shift($args));
    if (!empty($args)) {
      $count = array_shift($args);
    }
    if (!isset($count) || !is_numeric($count)) {
      $count = 1;
    }

    $nids = array_map('intval', $nids);
    foreach ($nids as $nid) {
      $node = node_load($nid);
      for ($i = 0; $i < $count; $i++) {
        $clone = drupal_clone($node);
        $clone->nid = $clone->vid = NULL;
        // If field to modify available is available, then add some random value.
        if ($modify_field && isset($clone->$modify_field)
          && (is_string($clone->$modify_field) || is_numeric($clone->$modify_field))) {
          $clone->$modify_field = is_numeric($clone->$modify_field)
            ? $clone->$modify_field + rand(1000, 9999)
            : $clone->$modify_field . rand(1000, 9999);
        }
        node_save($clone);
        $new_nids[] = $clone->nid;
      }
    }
  }

  if (!empty($new_nids)) {
    drush_print('New nodes created: ' . implode(', ', $new_nids));
  }
  else {
    drush_print('Nothing to clone.');
  }
}

/**
 * Drush callback
 * Delete nodes by nids.
 */
function drush_angil_node_delete() {
  $args = drush_get_arguments();
  // Skip op name.
  array_shift($args);
  $nids_deleted = array();
  if (!empty($args)) {
    $nids = explode(',', array_shift($args));
    $nids = array_map('intval', $nids);

    foreach ($nids as $nid) {
      $node = node_load($nid);
      if ($node) {
        _angil_node_delete($nid);
        $nids_deleted[] = $nid;
      }
    }
  }

  $count = count($nids_deleted);
  if ($count) {
    $message = ($count == 1)
      ? 'Node with nid ' . array_shift($nids_deleted) . ' was deleted.'
      : 'This nids were deleted: ' . implode(', ', $nids_deleted);
    drush_print($message);
  }
  else {
    drush_print('Nothing to delete.');
  }
}

/**
 * Drush callback
 * Publish nodes by nids.
 */
function drush_angil_node_publish() {
  $args = drush_get_arguments();
  // Skip op name.
  array_shift($args);
  _angil_change_node_status($args);
}

/**
 * Drush callback
 * Unpublish nodes by nids.
 */
function drush_angil_node_unpublish() {
  $args = drush_get_arguments();
  // Skip op name.
  array_shift($args);
  _angil_change_node_status($args, 0);
}

/**
 * Helper to proceed nodes status change operation.
 */
function _angil_change_node_status($args, $new_status = 1) {
  if (!empty($args)) {
    $already_proceeded = $proceeded = array();
    $new_status_name_past = $new_status ? 'published' : 'unpublished';
    $new_status_name = $new_status ? 'publish' : 'unpublish';
    $nids = explode(',', array_shift($args));
    $nids = array_map('intval', $nids);

    foreach ($nids as $nid) {
      $node = node_load($nid);
      if ($node) {
        if ($node->status != $new_status) {
          $node->status = $new_status;
          node_save($node);
          $proceeded[] = $node->nid;
        }
        else {
          $already_proceeded[] = $node->nid;
        }
      }
    }
    $count = count($proceeded);
    if ($count) {
      $message = format_plural(
        $count,
        'Node with nid @nid was !new_status_name.',
        'This nids were !new_status_name: @nids',
        array(
          '@nid' => array_shift($t = $proceeded),
          '@nids' => implode(', ', $proceeded),
          '!new_status_name' => $new_status_name_past,
        )
      );
      drush_print($message);
    }
    else {
      drush_print("Nothing to $new_status_name.");
    }

    $count = count($already_proceeded);
    if ($count) {
      $message = format_plural(
        $count,
        'Node with nid @nid already !new_status_name.',
        'This nids were already !new_status_name: @nids',
        array(
          '@nid' => array_shift($t = $already_proceeded),
          '@nids' => implode(', ', $already_proceeded),
          '!new_status_name' => $new_status_name_past,
        )
      );
      drush_print($message);
    }
  }
}

/**
 * Drush callback
 * Make a drush_print of node object by nid
 */
function drush_angil_node_dump() {
  $args = drush_get_arguments();
  // Skip op name.
  array_shift($args);
  $nid = array_shift($args);
  if (is_numeric($nid)) {
    $node = node_load($nid);
    if ($node) {
      var_dump($node);
      return;
    }
  }
  drush_print("Node with nid $nid not found");
}

/**
 * Drush callback
 * Show entered timestamp as date.
 */
function drush_angil_time_to_date() {
  $args = drush_get_arguments();
  // Skip op name.
  array_shift($args);
  $timestamp = array_shift($args);
  drush_print(format_date($timestamp, 'custom', 'd-m-Y H:i:s'));
}

/**
 * Drush callback
 * Clear content cache (panes, views, etc..).
 */
function drush_angil_clear_content_cache() {
  $core = array("cache", "cache_block", "cache_filter", "cache_page");
  $cache_tables = array_merge(module_invoke_all("flush_caches"), $core);
  foreach ($cache_tables as $table) {
    cache_clear_all("*", $table, TRUE);
  }
  drush_print('Cache cleared for ' . implode(', ', $cache_tables) . ' tables.');
}

/**
 * Drush callback
 * Get theming preprocesses list.
 */
function drush_angil_get_theme_preprosseces() {
  $args = drush_get_arguments();
  // Skip op name.
  array_shift($args);
  $hook = array_shift($args);

  // Ensure that theme initialized.
  init_theme();
  $hooks = theme_get_registry();

  if (!isset($hooks[$hook])) {
    drush_print('Theming not found');
    return FALSE;
  }

  $info = $hooks[$hook];
  if (isset($info['function'])) {
    drush_print('Theme hook implemented via function and have no preprocess.');
    return FALSE;
  }
  else {
    if (isset($info['preprocess functions']) && is_array($info['preprocess functions'])) {
      drush_print("Preprocesses for theme hook '$hook':");
      $i = 1;
      foreach ($info['preprocess functions'] as $preprocess_function) {
        drush_print("$i. $preprocess_function()");
        $i++;
      }
    }
  }
}

/**
 * Drush callback
 * Return list of specified modules ordered by weight.
 */
function drush_angil_compare_module_weight() {
  $args = drush_get_arguments();
  // Skip op name.
  array_shift($args);

  $modules_count = count($args);
  if ($modules_count == 1) {
    // Expect 'm1,m2,m3' syntax.
    $modules = explode(',', array_shift($args));
    $modules = array_map('trim', $modules);
  }
  elseif ($modules_count > 1) {
    // Expect 'm1 m2 m3' syntax.
    $modules = $args;
  }
  else {
    drush_print('No module names specified');
    return FALSE;
  }

  $query = db_query(
    'SELECT name, weight FROM {system} ' .
    'WHERE name IN (' . db_placeholders($modules, 'varchar') . ') AND type LIKE "module" ' .
    'ORDER BY weight ASC',
    $modules
  );

  $results = array();
  $i = 1;
  while ($res = db_fetch_object($query)) {
    $results[] = $res->name;
    drush_print("$i. " . drupal_ucfirst($res->name) . " ($res->weight)");
    $i++;
  }

  $missed_modules = array_diff($modules, $results);
  if (!empty($missed_modules)) {
    drush_print('Next modules are not found in system table: ' . implode(', ', $missed_modules));
  }
}

/**
 * Drush callback
 * Evaluate input string and var_dump() results.
 */
function drush_angil_vardump() {
  $args = drush_get_arguments();
  // Skip op name.
  array_shift($args);

  $expression = array_shift($args);
  $expression = "var_dump($expression);";
  // We don't use drupal_eval() because we want evaluate code in current context.
  eval($expression);
}

/**
 * List out broken records in system table.
 */
function drush_angil_check_system_table() {
  $query = db_query('SELECT filename, name FROM {system}');
  while ($res = db_fetch_object($query)) {
    if (!file_exists($res->filename)) {
      drush_print("File $res->filename for $res->name missed.");
    }
  }
}

/**
 * Return string, Flat representation of specified function. Usefull for debug on remote system.
 */
function drush_angil_get_flat_reflection() {
  if ($func_name = drush_get_option('func', FALSE)) {
    $reflection = ReflectionFunction::export($func_name, TRUE);
    if (strpos($reflection, '@@') !== FALSE) {
      $path_and_string = trim(array_pop(explode('@@', $reflection)));
      preg_match('/(.*?)\s(\d+)\s-\s(\d+)/', $path_and_string, $matches);
      list(, $path, $start, $end) = $matches;
      if ($content = @file_get_contents($path)) {
        $lines = explode("\n", $content);
        $func_code = implode("\n", array_slice($lines, $start, ($end - $start - 1)));
        $func_code = trim(str_replace("\n", ' ', $func_code));
        echo '<pre>';
        var_dump($func_code);
        echo '</pre>';
      }
    }
    else {
      drush_print('Coulnt get function reflection.');
    }
  }
}

/**
 * Drush callback
 * Check that path is registered in system.
 */
function drush_angil_path_check() {
  $return_item = drush_get_option('item', FALSE);

  $args = drush_get_arguments();
  // Skip op name.
  array_shift($args);

  if (!$args) {
    drush_print('Paths is not specified');
    return FALSE;
  }
  // Processing multiple arguments.
  foreach ($args as $path) {
    $item = _angil_path_check($path);
    if (!$item) {
      drush_print(dt('Path "!path" not exists', array('!path' => $path)));
      continue;
    }
    if ($return_item) {
      var_dump($item);
    }
    drush_print(dt('Path "!path" exists', array('!path' => $item['link_path'])));
  }
}

/**
 * Drush callback
 * Set username and password for uid = 1 .
 */
function drush_angil_admin_account() {
  $args = drush_get_arguments();
  // Skip op name.
  array_shift($args);

  if (!$args) {
    drush_print('Arguments not specified.');
    return FALSE;
  }
  $name = array_shift($args);
  $pass = array_shift($args);
  if ($pass) {
    $query = db_query(
      'UPDATE {users} SET name = "%s", pass = MD5("%s") WHERE uid = 1',
      $name,
      $pass
    );
    drush_print(dt('Admin account updated.'));
    drush_print(dt('Username: !name Password: !pass', array('!name' => $name, '!pass' => $pass)));
  }
  else {
    $query = db_query(
      'UPDATE {users} SET name = "%s" WHERE uid = 1',
      $name
    );
    drush_print(dt('Admin account updated.'));
    drush_print(dt('Username: !name', array('!name' => $name)));
  }
}
